﻿#region "SoapBox.Core License"
/// <header module="SoapBox.Core"> 
/// Copyright (C) 2009 SoapBox Automation Inc., All Rights Reserved.
/// Contact: SoapBox Automation Licencing (license@soapboxautomation.com)
/// 
/// This file is part of SoapBox Core.
/// 
/// Commercial Usage
/// Licensees holding valid SoapBox Automation Commercial licenses may use  
/// this file in accordance with the SoapBox Automation Commercial License
/// Agreement provided with the Software or, alternatively, in accordance 
/// with the terms contained in a written agreement between you and
/// SoapBox Automation Inc.
/// 
/// GNU Lesser General Public License Usage
/// SoapBox Core is free software: you can redistribute it and/or modify 
/// it under the terms of the GNU Lesser General Public License
/// as published by the Free Software Foundation, either version 3 of the
/// License, or (at your option) any later version.
/// 
/// SoapBox Core is distributed in the hope that it will be useful, 
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU Lesser General Public License for more details.
/// 
/// You should have received a copy of the GNU Lesser General Public License 
/// along with SoapBox Core. If not, see <http://www.gnu.org/licenses/>.
/// </header>
#endregion
        
using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using NUnit.Framework; 
using System.ComponentModel;

namespace SoapBox.Core
{
    [TestFixture]
    public class AbstractProgressBarTest
    {

        #region " Minimum "
        [Test]
        public void TestMinimum()
        {
            //test default value
            ConcreteProgressBar target = new ConcreteProgressBar();
            Assert.AreEqual(0, target.Minimum);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Minimum")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            double test = -15.3;
            target.SetMinimum(test);
            Assert.AreEqual(test, target.Minimum);
            Assert.IsTrue(propertyChanged);

        }
        #endregion

        #region " Maximum "
        [Test]
        public void TestMaximum()
        {
            //test default value
            ConcreteProgressBar target = new ConcreteProgressBar();
            Assert.AreEqual(100, target.Maximum);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Maximum")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            double test = -5.3;
            target.SetMaximum(test);
            Assert.AreEqual(test, target.Maximum);
            Assert.IsTrue(propertyChanged);

        }
        #endregion

        #region " Value "
        [Test]
        public void TestValue()
        {
            //test default value
            ConcreteProgressBar target = new ConcreteProgressBar();
            Assert.AreEqual(0, target.Value);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Value")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            double test = Math.PI;
            target.SetValue(test);
            Assert.AreEqual(test, target.Value);
            Assert.IsTrue(propertyChanged);

        }
        #endregion

    }
}
