﻿#region "SoapBox.Core License"
/// <header module="SoapBox.Core"> 
/// Copyright (C) 2009 SoapBox Automation Inc., All Rights Reserved.
/// Contact: SoapBox Automation Licencing (license@soapboxautomation.com)
/// 
/// This file is part of SoapBox Core.
/// 
/// Commercial Usage
/// Licensees holding valid SoapBox Automation Commercial licenses may use  
/// this file in accordance with the SoapBox Automation Commercial License
/// Agreement provided with the Software or, alternatively, in accordance 
/// with the terms contained in a written agreement between you and
/// SoapBox Automation Inc.
/// 
/// GNU Lesser General Public License Usage
/// SoapBox Core is free software: you can redistribute it and/or modify 
/// it under the terms of the GNU Lesser General Public License
/// as published by the Free Software Foundation, either version 3 of the
/// License, or (at your option) any later version.
/// 
/// SoapBox Core is distributed in the hope that it will be useful, 
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU Lesser General Public License for more details.
/// 
/// You should have received a copy of the GNU Lesser General Public License 
/// along with SoapBox Core. If not, see <http://www.gnu.org/licenses/>.
/// </header>
#endregion
        
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;
using System.ComponentModel;
using System.Collections.ObjectModel;

namespace SoapBox.Core
{
    [TestFixture]
    public class AbstractDocumentTest
    {
        #region " Memento "
        [Test]
        public void TestMemento()
        {
            //test default value
            ConcreteDocument target = new ConcreteDocument();
            Assert.AreEqual(string.Empty, target.Memento);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Memento")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            string test = Guid.NewGuid().ToString();
            target.SetMemento(test);
            Assert.AreEqual(test, target.Memento);
            Assert.IsTrue(propertyChanged);
        }

        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestMementoNull()
        {
            //should throw exception on null value
            ConcreteDocument target = new ConcreteDocument();
            target.SetMemento(null);
        }
        #endregion

        #region " CreateDocument "
        [Test]
        public void TestCreateDocument()
        {
            //test default value
            ConcreteDocument target = new ConcreteDocument();
            Assert.AreEqual(string.Empty, target.Memento);
            IDocument result = target.CreateDocument("testVal");
            Assert.AreSame(target, result);
            Assert.AreEqual(target.Memento, "testVal");
            Assert.AreEqual(result.Memento, "testVal");
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCreateDocumentTwice()
        {
            ConcreteDocument target = new ConcreteDocument();
            IDocument result1 = target.CreateDocument("testVal1");
            IDocument result2 = target.CreateDocument("testVal2");
        }
        #endregion

        #region "OnClosed, OnClosing, OnOpened"
        [Test]
        public void Can_call_OnClosed_OnClosing_OnOpened()
        {
            ConcreteDocument target = new ConcreteDocument();
            target.OnOpened(this, EventArgs.Empty);
            target.OnClosing(this, new CancelEventArgs());
            target.OnClosed(this, EventArgs.Empty);
        }
        #endregion

        [Test]
        public void TestDocumentAttribute()
        {
            var target = new DocumentAttribute();
        }

        [Test]
        public void TestPadAttribute()
        {
            var target = new PadAttribute();
        }
    }
}
