﻿#region "SoapBox.Core License"
/// <header module="SoapBox.Core"> 
/// Copyright (C) 2009 SoapBox Automation Inc., All Rights Reserved.
/// Contact: SoapBox Automation Licencing (license@soapboxautomation.com)
/// 
/// This file is part of SoapBox Core.
/// 
/// Commercial Usage
/// Licensees holding valid SoapBox Automation Commercial licenses may use  
/// this file in accordance with the SoapBox Automation Commercial License
/// Agreement provided with the Software or, alternatively, in accordance 
/// with the terms contained in a written agreement between you and
/// SoapBox Automation Inc.
/// 
/// GNU Lesser General Public License Usage
/// SoapBox Core is free software: you can redistribute it and/or modify 
/// it under the terms of the GNU Lesser General Public License
/// as published by the Free Software Foundation, either version 3 of the
/// License, or (at your option) any later version.
/// 
/// SoapBox Core is distributed in the hope that it will be useful, 
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU Lesser General Public License for more details.
/// 
/// You should have received a copy of the GNU Lesser General Public License 
/// along with SoapBox Core. If not, see <http://www.gnu.org/licenses/>.
/// </header>
#endregion
        
using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using NUnit.Framework; 
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Windows.Media.Imaging;
using System.Windows.Controls;
using System.Drawing;
using System.Windows.Media;
using System.Windows;

namespace SoapBox.Core
{
    [TestFixture]
    public class AbstractLayoutItemTest
    {

        #region " Name "
        [Test]
        public void TestName()
        {
            //test default value
            ConcreteLayoutItem target = new ConcreteLayoutItem();
            Assert.AreEqual(string.Empty, target.Name);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
                {
                    if (e.PropertyName == "Name")
                    {
                        Assert.IsFalse(propertyChanged);
                        propertyChanged = true;
                    }
                    else
                    {
                        throw new Exception("Unexpected Property Changed.");
                    }
                };
            string test = Guid.NewGuid().ToString();
            target.SetName(test);
            Assert.AreEqual(test, target.Name);
            Assert.IsTrue(propertyChanged);
        }

        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestNameNull()
        {
            //should throw exception on null value
            ConcreteLayoutItem target = new ConcreteLayoutItem();
            target.SetName(null);
        }
        #endregion

        #region " Title "
        [Test]
        public void TestTitle()
        {
            //test default value
            ConcreteLayoutItem target = new ConcreteLayoutItem();
            Assert.AreEqual(string.Empty, target.Title);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Title")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            string test = Guid.NewGuid().ToString();
            target.SetTitle(test);
            Assert.AreEqual(test, target.Title);
            Assert.IsTrue(propertyChanged);
        }

        [Test]
        public void Setting_Title_to_null_throws()
        {
            ConcreteLayoutItem target = new ConcreteLayoutItem();
            Assert.Throws<ArgumentNullException>(() => target.SetTitle(null));
        }
        #endregion

        #region " Icon "
        [Test]
        public void TestIcon()
        {
            //test default value
            var target = new ConcreteLayoutItem();
            Assert.IsNull(target.Icon);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Icon")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            BitmapSource img = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(
                (new Bitmap(1, 1)).GetHbitmap(),
                IntPtr.Zero,
                Int32Rect.Empty,
                System.Windows.Media.Imaging.BitmapSizeOptions.FromEmptyOptions());
            target.SetIcon(img);
            Assert.AreEqual(img, target.Icon);
            Assert.IsTrue(propertyChanged);

            propertyChanged = false;
            target.SetIcon(null);
            Assert.IsNull(target.Icon);
            Assert.IsTrue(propertyChanged);

            propertyChanged = false;
            Bitmap b = new Bitmap(1, 1);
            target.SetIconFromBitmap(b);
            Assert.AreNotEqual(img, target.Icon);
            Assert.IsNotNull(target.Icon);
            Assert.IsTrue(propertyChanged);

        }
        #endregion

        #region "OnGotFocus, OnLostFocus"
        [Test]
        public void Can_Call_OnGotFocus_OnLostFocus()
        {
            ConcreteLayoutItem target = new ConcreteLayoutItem();
            target.OnGotFocus(this, new RoutedEventArgs());
            target.OnLostFocus(this, new RoutedEventArgs());
        }
        #endregion

        [Test]
        public void TestLayoutItemAttribute()
        {
            var target = new LayoutItemAttribute();
            Assert.IsNull(target.Name);
            target.Name = string.Empty;
            Assert.AreEqual(string.Empty, target.Name);
        }


    }
}
