﻿#region "SoapBox.Core License"
/// <header module="SoapBox.Core"> 
/// Copyright (C) 2009 SoapBox Automation Inc., All Rights Reserved.
/// Contact: SoapBox Automation Licencing (license@soapboxautomation.com)
/// 
/// This file is part of SoapBox Core.
/// 
/// Commercial Usage
/// Licensees holding valid SoapBox Automation Commercial licenses may use  
/// this file in accordance with the SoapBox Automation Commercial License
/// Agreement provided with the Software or, alternatively, in accordance 
/// with the terms contained in a written agreement between you and
/// SoapBox Automation Inc.
/// 
/// GNU Lesser General Public License Usage
/// SoapBox Core is free software: you can redistribute it and/or modify 
/// it under the terms of the GNU Lesser General Public License
/// as published by the Free Software Foundation, either version 3 of the
/// License, or (at your option) any later version.
/// 
/// SoapBox Core is distributed in the hope that it will be useful, 
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU Lesser General Public License for more details.
/// 
/// You should have received a copy of the GNU Lesser General Public License 
/// along with SoapBox Core. If not, see <http://www.gnu.org/licenses/>.
/// </header>
#endregion
        
using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using NUnit.Framework; 
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Windows.Media.Imaging;
using System.Windows.Controls;
using System.Drawing;
using System.Windows.Media;

namespace SoapBox.Core
{
    [TestFixture]
    public class AbstractOptionsPadTest
    {

        #region " CommitChanges "
        [Test]
        public void TestCommitNoChanges()
        {
            ConcreteOptionsPad target = new ConcreteOptionsPad();
            target.Commit();
        }
        [Test]
        public void TestCommitChange()
        {
            Properties.Settings.Default.SomeProperty = false;
            ConcreteOptionsPad target = new ConcreteOptionsPad();
            target.SomeProperty = true;
            Assert.IsTrue(target.SomeProperty);
            target.Commit();
            Assert.IsTrue(target.SomeProperty);
            Assert.IsTrue(Properties.Settings.Default.SomeProperty);
        }
        #endregion

        #region " CancelChanges "
        [Test]
        public void TestCancelNoChanges()
        {
            ConcreteOptionsPad target = new ConcreteOptionsPad();
            target.Cancel();
        }
        [Test]
        public void TestCancelChange()
        {
            Properties.Settings.Default.SomeProperty = false;
            ConcreteOptionsPad target = new ConcreteOptionsPad();
            target.SomeProperty = true;
            Assert.IsTrue(target.SomeProperty);
            target.Cancel();
            Assert.IsFalse(target.SomeProperty);
            Assert.IsFalse(Properties.Settings.Default.SomeProperty);
        }
        #endregion

        #region " OptionChanged "
        [Test]
        public void TestOptionChanged()
        {
            ConcreteOptionsPad target = new ConcreteOptionsPad();

            int optionChangedCount = 0;
            target.OptionChanged += new EventHandler((s, e) =>
            {
                optionChangedCount++;
            });

            target.TestNotifyOptionChanged();
            Assert.AreEqual(1, optionChangedCount);
        }
        #endregion
    }
}
