#region "SoapBox.Core License"
/// <header module="SoapBox.Core"> 
/// Copyright (C) 2009 SoapBox Automation Inc., All Rights Reserved.
/// Contact: SoapBox Automation Licencing (license@soapboxautomation.com)
/// 
/// This file is part of SoapBox Core.
/// 
/// Commercial Usage
/// Licensees holding valid SoapBox Automation Commercial licenses may use  
/// this file in accordance with the SoapBox Automation Commercial License
/// Agreement provided with the Software or, alternatively, in accordance 
/// with the terms contained in a written agreement between you and
/// SoapBox Automation Inc.
/// 
/// GNU Lesser General Public License Usage
/// SoapBox Core is free software: you can redistribute it and/or modify 
/// it under the terms of the GNU Lesser General Public License
/// as published by the Free Software Foundation, either version 3 of the
/// License, or (at your option) any later version.
/// 
/// SoapBox Core is distributed in the hope that it will be useful, 
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU Lesser General Public License for more details.
/// 
/// You should have received a copy of the GNU Lesser General Public License 
/// along with SoapBox Core. If not, see <http://www.gnu.org/licenses/>.
/// </header>
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;
using System.ComponentModel;
using Rhino.Mocks;
using System.Collections.ObjectModel;
using System.Windows.Media.Imaging;
using System.Drawing;
using System.Windows;

namespace SoapBox.Core
{
    [TestFixture]
    public class AbstractTreeViewItemTest
    {
        [Test]
        public void Test_IsSelected()
        {
            //test default value
            var target = new ConcreteTreeViewItem();
            Assert.IsFalse(target.IsSelected);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "IsSelected")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };

            bool isSelectedChanged = false;
            target.IsSelectedChanged += delegate(object sender, EventArgs e)
            {
                Assert.IsFalse(isSelectedChanged);
                isSelectedChanged = true;
            };

            target.IsSelected = true;
            Assert.IsTrue(target.IsSelected);
            Assert.IsTrue(propertyChanged);
            Assert.IsTrue(isSelectedChanged);
        }

        [Test]
        public void Test_IsExpanded()
        {
            //test default value
            var target = new ConcreteTreeViewItem();
            Assert.IsFalse(target.IsExpanded);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "IsExpanded")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };

            target.IsExpanded = true;
            Assert.IsTrue(target.IsExpanded);
            Assert.IsTrue(propertyChanged);
        }

        [Test]
        public void Test_Items()
        {
            //test default value
            var target = new ConcreteTreeViewItem();
            Assert.IsNull(target.Items);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Items")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };

            var newItems = MockRepository.GenerateStub<IEnumerable<ITreeViewItem>>();
            target.SetItems(newItems);
            Assert.AreSame(newItems, target.Items);
            Assert.IsTrue(propertyChanged);
        }

        [Test]
        public void Test_Icons()
        {
            //test default value
            var target = new ConcreteTreeViewItem();
            Assert.IsNotNull(target.Icons);
            Assert.AreEqual(0, target.Icons.Count);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "Icons")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };

            var newIcons = MockRepository.GenerateStub<ObservableCollection<object>>();
            target.SetIcons(newIcons);
            Assert.AreSame(newIcons, target.Icons);
            Assert.IsTrue(propertyChanged);
        }

        [Test, RequiresSTA]
        public void Test_AddIconFromBitmap()
        {
            var target = new ConcreteTreeViewItem();
            Assert.AreEqual(0, target.Icons.Count);
            Bitmap b = new Bitmap(1, 1);

            var result = target.AddIconFromBitmap(b);

            Assert.AreEqual(1, target.Icons.Count);
            var img = target.Icons[0] as System.Windows.Controls.Image;
            Assert.IsNotNull(img);
            Assert.AreSame(result, img);
        }

    }
}
