﻿#region "SoapBox.Core License"
/// <header module="SoapBox.Core"> 
/// Copyright (C) 2009 SoapBox Automation Inc., All Rights Reserved.
/// Contact: SoapBox Automation Licencing (license@soapboxautomation.com)
/// 
/// This file is part of SoapBox Core.
/// 
/// Commercial Usage
/// Licensees holding valid SoapBox Automation Commercial licenses may use  
/// this file in accordance with the SoapBox Automation Commercial License
/// Agreement provided with the Software or, alternatively, in accordance 
/// with the terms contained in a written agreement between you and
/// SoapBox Automation Inc.
/// 
/// GNU Lesser General Public License Usage
/// SoapBox Core is free software: you can redistribute it and/or modify 
/// it under the terms of the GNU Lesser General Public License
/// as published by the Free Software Foundation, either version 3 of the
/// License, or (at your option) any later version.
/// 
/// SoapBox Core is distributed in the hope that it will be useful, 
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU Lesser General Public License for more details.
/// 
/// You should have received a copy of the GNU Lesser General Public License 
/// along with SoapBox Core. If not, see <http://www.gnu.org/licenses/>.
/// </header>
#endregion
        
using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using NUnit.Framework; 
using System.ComponentModel;

namespace SoapBox.Core
{
    [TestFixture]
    public class AbstractExtensionTest
    {

        #region " ID "
        [Test]
        public void TestID()
        {
            //test default value
            ConcreteExtension target = new ConcreteExtension();
            Guid id = new Guid(target.ID);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "ID")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            string test = Guid.NewGuid().ToString();
            target.SetID(test);
            Assert.AreEqual(test, target.ID);
            Assert.IsTrue(propertyChanged);
        }

        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIDNull()
        {
            //should throw exception on null value
            ConcreteExtension target = new ConcreteExtension();
            target.SetID(null);
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIDEmpty()
        {
            //should throw exception on empty value
            ConcreteExtension target = new ConcreteExtension();
            target.SetID(string.Empty);
        }
        #endregion

        #region " InsertRelativeToID "
        [Test]
        public void TestInsertRelativeToID()
        {
            //test default value
            ConcreteExtension target = new ConcreteExtension();
            Assert.IsNull(target.InsertRelativeToID);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "InsertRelativeToID")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            string test = Guid.NewGuid().ToString();
            target.SetInsertRelativeToID(test);
            Assert.AreEqual(test, target.InsertRelativeToID);
            Assert.IsTrue(propertyChanged);

            propertyChanged = false;
            target.SetInsertRelativeToID(null);
            Assert.IsNull(target.InsertRelativeToID);
            Assert.IsTrue(propertyChanged);
        }
        #endregion

        #region " BeforeOrAfter "
        [Test]
        public void TestBeforeOrAfter()
        {
            //test default value
            ConcreteExtension target = new ConcreteExtension();
            Assert.AreEqual(RelativeDirection.Before, target.BeforeOrAfter);

            //test setting value and property changed event
            bool propertyChanged = false;
            target.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == "BeforeOrAfter")
                {
                    Assert.IsFalse(propertyChanged);
                    propertyChanged = true;
                }
                else
                {
                    throw new Exception("Unexpected Property Changed.");
                }
            };
            target.SetBeforeOrAfter(RelativeDirection.After);
            Assert.AreEqual(RelativeDirection.After, target.BeforeOrAfter);
            Assert.IsTrue(propertyChanged);
        }
        #endregion


    }
}
